<?php
/**
 * Load add-ons file.
 *
 * @package WishListMember\Features\Addons_Manager
 */

namespace WishListMember\Features\Addons_Manager;

use WP_Error;

/**
 * Addons class
 */
class Addons {
	/**
	 * Our own instance.
	 *
	 * @var Addons
	 */
	private static $instance = null;

	/**
	 * Return instasnce of this class
	 *
	 * @return Addons
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Get add-ons for this specific WishList Member product
	 *
	 * @return array
	 */
	public function get_addons() {
		$arg_array = array(
			'method'    => 'GET',
			'timeout'   => 15,
			'blocking'  => true,
			'sslverify' => false,
		);
		$uri       = str_replace(
			array( '{product_slug}', '{license_key}', '{domain}' ),
			array( WLM_PRODUCT_SLUG, wishlistmember_instance()->get_option( 'LicenseKey' ), wp_parse_url( get_bloginfo( 'url' ), PHP_URL_HOST ) ),
			WLM_ADDONS_MANAGER_URI
		);
		$resp      = wp_remote_request( $uri, $arg_array );
		return (array) json_decode( $resp['body'] );
	}


	/**
	 * Install add-on
	 *
	 * @param string $addon_url Add-on url.
	 * @return array|false Array containing installation status or false on failure.
	 */
	public function install_addon( $addon_url ) {
		// lets included necessary files.
		require_once ABSPATH . 'wp-admin/includes/plugin-install.php';
		require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
		require_once ABSPATH . 'wp-admin/includes/class-wp-ajax-upgrader-skin.php';
		require_once ABSPATH . 'wp-admin/includes/class-plugin-upgrader.php';
		// install the plugin.
		$skin      = new \WP_Ajax_Upgrader_Skin();
		$installer = new \Plugin_Upgrader( $skin );
		$plugin    = wp_unslash( $addon_url );
		$installer->install( $plugin );
		wp_cache_flush();
		// installation successfull if plugin details is available.
		if ( $installer->plugin_info() ) {
			// lets try activating it.
			$plugin_basename = $installer->plugin_info();
			$activated       = activate_plugin( $plugin_basename );
			if ( ! is_wp_error( $activated ) ) {
				return array(
					'message'   => __( 'Plugin installed & activated.', 'wishlist-member' ),
					'activated' => true,
					'basename'  => $plugin_basename,
				);
			} else {
				return array(
					'message'   => __( 'Plugin installed.', 'wishlist-member' ),
					'activated' => false,
					'basename'  => $plugin_basename,
				);
			}
		}
		return false;
	}

	/**
	 * Active add-on
	 *
	 * @param string $addon_file Add-on file.
	 * @return boolean | WP_Error
	 */
	public function activate_addon( $addon_file ) {
		return activate_plugin( wp_unslash( $addon_file ) );
	}

	/**
	 * Deactivate add-on
	 *
	 * @param string $addon_file Add-on file.
	 * @return true
	 */
	public function deactivate_addon( $addon_file ) {
		deactivate_plugins( wp_unslash( $addon_file ) );
		return true;
	}
}
