<?php
/**
 * Settings
 *
 * @package WishListMember/Features/Addons_Manager
 */

namespace WishListMember\Features\Addons_Manager;

add_filter( 'wishlist_member_menu', __NAMESPACE__ . '\settings_menu', 1 );
/**
 * Add Add-ons Manager menu
 * Called by 'wishlist_member_menu' hook
 *
 * @param  array $menus WishList Member menu items.
 * @return array
 */
function settings_menu( $menus ) {
	$menus[] = array(
		'key'   => 'addons',
		'name'  => __( 'Add-ons', 'wishlist-member' ),
		'title' => __( 'WishList Member Add-ons', 'wishlist-member' ),
		'icon'  => 'open_in_browser',
		'sub'   => array(),
	);
	return $menus;
}

add_action( 'wishlistmember_pre_admin_screen', __NAMESPACE__ . '\settings_scripts_and_styles', 10 );
/**
 * Load view styles and scripts.
 * Called by 'wishlistmember_pre_admin_screen' hook.
 *
 * @param  string $wl Current view being requested.
 */
function settings_scripts_and_styles( $wl ) {
	$css_handle = 'wishlist-member-addons-manager';
	$css_file   = __DIR__ . '/views/admin/settings.css';
	wp_enqueue_style(
		$css_handle,
		plugins_url( basename( $css_file ), $css_file ),
		array(),
		WLM_PLUGIN_VERSION
	);
	$js_handle = 'wishlist-member-addons-manager';
	$js_file   = __DIR__ . '/views/admin/settings.js';
	wp_enqueue_script(
		$js_handle,
		plugins_url( basename( $js_file ), $js_file ),
		array(),
		WLM_PLUGIN_VERSION,
		false
	);
	wp_print_styles( $css_handle );
	wp_print_scripts( $js_handle );
}

add_action( 'wishlistmember_admin_screen', __NAMESPACE__ . '\load_settings_screens' );
/**
 * Display admin views
 * Called by 'wishlistmember_admin_screen' hook
 *
 * @param  string $wl Current view being requested.
 */
function load_settings_screens( $wl ) {
	switch ( $wl ) {
		case 'addons':
			require_once __DIR__ . '/views/admin/settings.php';
			break;
	}
}

add_action( 'wp_ajax_wlm_toggle_enable_addon', __NAMESPACE__ . '\toggle_enable_addon' );
/**
 * `wp_ajax_wlm_toggle_enable_addon` handler to enable/disable an add-on.
 */
function toggle_enable_addon() {
	$addon_file = wlm_or( wlm_arrval( wlm_post_data( true ), 'addon_file' ), '' );
	$enable     = (int) wlm_or( wlm_arrval( wlm_post_data( true ), 'enable' ), '' );
	$array_ret  = array( 'enabled' => $enable );
	if ( 1 === $enable ) {
		$ret = \WishListMember\Features\Addons_Manager\Addons::instance()->activate_addon( $addon_file );
		if ( ! is_wp_error( $ret ) ) {
			$array_ret['message'] = 'Addon enabled';
			wp_send_json_success( $array_ret );
		} else {
			$array_ret['message'] = 'An error occured while enabling the addon. ' . $ret->get_error_message();
			wp_send_json_error( $array_ret );
		}
	} else {
		if ( \WishListMember\Features\Addons_Manager\Addons::instance()->deactivate_addon( $addon_file ) ) {
			$array_ret['message'] = 'Addon disabled';
			wp_send_json_success( $array_ret );
		} else {
			// prepare this part in case we change the deactivate_addon later.
			$array_ret['message'] = 'An error occured while disabling the addon';
			wp_send_json_error( $array_ret );
		}
	}
}

add_action( 'wp_ajax_wlm_install_addon', __NAMESPACE__ . '\install_addon' );
/**
 * `wp_ajax_wlm_install_addon` handler to install an add-on.
 */
function install_addon() {
	$addon_url = isset( wlm_post_data()['addon_url'] ) ? wlm_post_data()['addon_url'] : '';
	$ret       = \WishListMember\Features\Addons_Manager\Addons::instance()->install_addon( $addon_url );
	$array_ret = array( 'addon_url' => $addon_url );
	if ( is_array( $ret ) ) {
		$array_ret['message'] = $ret['message'];
		wp_send_json_success( $array_ret );
	} else {
		$array_ret['message'] = __( 'An error occured while installing the addon', 'wishlist-member' );
		wp_send_json_error( $array_ret );
	}
}
